class Mode:
    NORMAL = 'normal'
    BOLD = 'bold'
    UNDERLINE = 'underline'
    BLINK = 'blink'
    INVERT = 'invert'
    HIDE = 'hide'

    def __repr__(self):
        return '123'

    def __str__(self):
        return '123'


print(Mode)



import re
import os
import functools
import sys
import threading
from copy import deepcopy
from inspect import isfunction
from pprint import pprint
from queue import Queue
from colour_printing.style import setting
from colour_printing.custom.config import Config

stream = sys.stdout
level_list = []


def level_wrap(func):
    level_list.append(func.__name__.upper())

    @functools.wraps(func)
    def wrap(self, *args, **kwargs):
        level = func.__name__.upper()
        default = self.default.get(level, {})
        data = {}
        # 参数
        for i in self.term:
            w = kwargs.get(i)
            if w:
                data[i] = w
            else:
                dd = default.get(i, lambda: "")
                if isfunction(dd):
                    data[i] = dd()
                else:
                    data[i] = dd
        sep = kwargs.get('sep', " ")
        data['message'] = sep.join([str(i) for i in args])
        # record
        self.handler_record(deepcopy(data))
        # 最高优先级,对data的操作会影响后续操作的输出内容
        res = func(self, data)
        # 日志
        if level not in self.log_filter:
            msg = self.raw_template.format(**data) + "\n"
            self.queue.put(msg)
        # 打印
        if self.switch and self.Master_switch and level not in self.print_filter:
            end = kwargs.get('end', '\n')
            self.show(level, data=data, end=end)
        return res

    return wrap


class PrintMeError(Exception):
    def __init__(self, message):
        super().__init__(f"\n[*]Tip>> {message}")


class ColourPrinting(object):
    Master_switch = True

    @level_wrap
    def info(self, data):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    @level_wrap
    def debug(self, data):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    @level_wrap
    def error(self, data):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    @level_wrap
    def warning(self, data):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    @level_wrap
    def success(self, data):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    def handler_record(self, record):
        """record不受level函数影响,处理日志信息 重写此函数以应用每个不同的使用场景 """


class LogHandler(object):
    def __init__(self, printme):
        self.printme = printme
        self.log_path = os.getcwd()
        self.log_name = 'colour_printing_log.log'
        self.log_delay = 60 * 2
        self.main_t = None

    def run(self, log_path: str = '', log_name: str = ''):  # log
        """
        :param log_path: 路径
        :param log_name: 文件名
        :return:
        """
        if log_path:
            self.log_path = log_path
        if log_name:
            self.log_name = log_name
        self.main_t = threading.currentThread()
        t = threading.Thread(target=self.__log_to_file, args=())
        t.start()

    def __log_to_file(self):
        path = os.path.join(self.log_path, self.log_name)
        stream.write(f'[*]Tip>> 日志文件path: {path}\n')
        with open(path, 'a+') as f:
            while True:
                if self.printme.queue.empty():
                    if self.main_t.isAlive():
                        continue
                    else:
                        break
                f.write(self.printme.queue.get())
                f.flush()


def make_default():
    r = {
        "DEFAULT": lambda: "",  # 默认值<-- Must be function name or lambda expression
        "fore": "blue",  # 前景色
        "back": "",  # 背景色
        "mode": "",  # 模式
    }
    return r


def make_level_default(term):
    r = {}
    for t in term:
        r.update({t: make_default()})
    return r


class PrintMe(ColourPrinting):

    def __init__(self, **kwargs):
        self.raw_template = ''
        self.term = []
        self.template = ''
        # store
        self.level_list = level_list
        self.box = {}
        self.default = {}
        # switch
        self.__switch = True
        self.__print_filter = []
        self.__log_filter = []
        # style config
        self.config = Config(printme=self)
        # log
        self.queue = Queue()
        self.log_handler = LogHandler(printme=self)
        for k, v in kwargs.items():
            if k in dir(self):
                raise PrintMeError(f'变量名"{k}"已被定义,请更换')
            setattr(self, k, v)

    def load_config(self):
        """获取py文件中的配置"""
        # template 检查
        template = self.config.get('TEMPLATE')
        if not template:
            raise PrintMeError(f"'{self.config.filename}' Can't find TEMPLATE = '' ")
        self.raw_template = template
        self.term = re.findall(r'(?<=\{)[^}]*(?=\})+', template)
        for t in self.term:
            if t.strip() == '':
                raise PrintMeError('Template have {} ! ')
        if "message" not in self.term:
            raise PrintMeError('Template muse have {message} ! ')
        term_wrap = {i: "{%s}{%s}{%s}" % (i + '0', i, i + '1') for i in self.term}
        self.template = template.format(**term_wrap)
        # style map
        for level in self.level_list:
            default = self.default[level] = {}
            style = self.box[level] = {}
            terms = self.config.get(level, make_level_default(self.term))  # 获取每个level里的字段
            for t in self.term:
                cfg = terms.get(t, make_default())  # 获取每个字段的default,fore,mode,back
                default.update({t: cfg.get("DEFAULT")})
                sett = setting(fore=cfg.get('fore'), back=cfg.get('back'), mode=cfg.get('mode'))
                style.update({f'{t}0': sett[0], f'{t}1': sett[1]})

    def show(self, level: str, data: dict, end: str):
        # style
        if not self.box:
            msg = self.raw_template.format(**data)
        else:
            style = self.box.get(level.upper())
            data.update(style)
            msg = self.template.format(**data)
        stream.write(msg)
        stream.write(end)

    def set_default(self, **kwargs):
        """
        设置默认值
        :param kwargs:
        :return:
        """
        level = kwargs.get('set_level')
        if level:
            if level.upper() in self.level_list:
                for k, v in kwargs.items():
                    self.default[level.upper()].update({k: v})
        else:
            for level in self.level_list:
                for k, v in kwargs.items():
                    self.default[level.upper()].update({k: v})

    @property
    def switch(self):
        return self.__switch

    @switch.setter
    def switch(self, val):
        self.__switch = val

    @property
    def print_filter(self):
        return self.__print_filter

    @print_filter.setter
    def print_filter(self, val):
        if not isinstance(val, list):
            val = [val]
        self.__print_filter = [i.upper() for i in val]

    @property
    def log_filter(self):
        return self.__log_filter

    @log_filter.setter
    def log_filter(self, val):
        if not isinstance(val, list):
            val = [val]
        self.__log_filter = [i.upper() for i in val]



"""
#                     *Colour-printing Reference*
#########################################################################################
#   @'fore': # 前景色         @'back':# 背景              @'mode':# 显示模式               # 
#            'black': 黑色            'black':  黑色              'normal': 终端默认设置   # 
#            'red': 红色              'red':  红色                'bold':  高亮显示        # 
#            'green': 绿色            'green': 绿色               'underline':  使用下划线 #
#            'yellow': 黄色           'yellow': 黄色              'blink': 闪烁           # 
#            'blue':  蓝色            'blue':  蓝色               'invert': 反白显示       #    
#            'purple':  紫红色        'purple':  紫红色            'hide': 不可见          #    
#            'cyan':  青蓝色          'cyan':  青蓝色                                     #
#            'white':  白色           'white':  白色                                     #
#########################################################################################
"""
from datetime import datetime
from colour_printing import Mode, Fore, Back

get_time = lambda: datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S.%f")[:-3]

TEMPLATE = "{time} {flag} {message}"

time_default = get_time

flag_default = ""

message_default = ""

fill = 7

INFO = {
    "time": {
        "DEFAULT": time_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.CYAN,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },

    "flag": {
        "DEFAULT": "INFO".center(fill, "-"),  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.BLUE,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode.INVERT,  # 模式
    },

    "message": {
        "DEFAULT": message_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.BLUE,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },
}

ERROR = {
    "time": {
        "DEFAULT": time_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.CYAN,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },

    "flag": {
        "DEFAULT": "ERROR".center(fill, "-"),  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.RED,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode.INVERT,  # 模式
    },

    "message": {
        "DEFAULT": message_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.RED,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },
}

SUCCESS = {
    "time": {
        "DEFAULT": time_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.CYAN,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },

    "flag": {
        "DEFAULT": "SUCCESS".center(fill, "-"),  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.GREEN,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode.INVERT,  # 模式
    },

    "message": {
        "DEFAULT": message_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.GREEN,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },
}

DEBUG = {
    "time": {
        "DEFAULT": time_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.CYAN,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },

    "flag": {
        "DEFAULT": "DEBUG".center(fill, "-"),  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.PURPLE,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode.INVERT,  # 模式
    },

    "message": {
        "DEFAULT": message_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.PURPLE,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },
}

WARNING = {
    "time": {
        "DEFAULT": time_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.CYAN,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },

    "flag": {
        "DEFAULT": "WARN".center(fill, "-"),  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.YELLOW,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode.INVERT,  # 模式
    },

    "message": {
        "DEFAULT": message_default,  # 默认值<-- Must be function name or lambda expression
        "fore": Fore.YELLOW,  # 前景色
        "back": Back,  # 背景色
        "mode": Mode,  # 模式
    },
}



import errno
import os
import sys
import types

stream = sys.stdout


def import_string(import_name, silent=False):
    import_name = str(import_name).replace(":", ".")
    try:
        try:
            __import__(import_name)
        except ImportError:
            if "." not in import_name:
                raise
        else:
            return sys.modules[import_name]

        module_name, obj_name = import_name.rsplit(".", 1)
        module = __import__(module_name, globals(), locals(), [obj_name])
        try:
            return getattr(module, obj_name)
        except AttributeError as e:
            raise ImportError(e)

    except ImportError as e:
        raise e


string_types = (str,)
integer_types = (int,)

"""从.py文件导入"""


class Config(dict):
    def __init__(self, printme=None):
        dict.__init__(self)
        self.printme = printme
        self.config_str = ""
        self.filename = ""

    def from_pyfile(self, file_path, silent=False):
        filename = os.path.split(file_path)[1]
        self.filename = filename
        d = types.ModuleType('config')
        d.__file__ = filename
        try:
            with open(file_path, mode='rb') as config_file:
                config = config_file.read()
                self.config_str = config.decode()
                exec(compile(config, filename, 'exec'), d.__dict__)
        except IOError as e:
            if silent and e.errno in (
                    errno.ENOENT, errno.EISDIR, errno.ENOTDIR
            ):
                return False
            e.strerror = 'Unable to load configuration file (%s)' % e.strerror
            raise
        return self.from_object(d)

    def from_object(self, obj):
        if isinstance(obj, string_types):
            obj = import_string(obj)
        for key in dir(obj):
            if key.isupper():
                self[key] = getattr(obj, key)
        if self.printme:
            self.printme.load_config()  # 渲染
        return True



import re
import sys
import argparse
import os
from colour_printing.custom.config import Config
from colour_printing import cword

stream = sys.stdout
parser = argparse.ArgumentParser()
parser.description = cword("***** Colour-printing ***** github@Faithforus ", fore="cyan")
parser.add_argument('-n', '--filename', default="colour_printing_config.py", help=u'配置文件名')
parser.add_argument('-t', '--template', help="输出信息模板")
parser.add_argument('-l', '--newLevel', help="新增level")

lk = '{'
rk = '}'
dy = "\"\""

header = """\"""
#                     *Colour-printing Reference*
#########################################################################################
#   @'fore': # 前景色         @'back':# 背景              @'mode':# 显示模式               # 
#            'black': 黑色            'black':  黑色              'normal': 终端默认设置   # 
#            'red': 红色              'red':  红色                'bold':  高亮显示        # 
#            'green': 绿色            'green': 绿色               'underline':  使用下划线 #
#            'yellow': 黄色           'yellow': 黄色              'blink': 闪烁           # 
#            'blue':  蓝色            'blue':  蓝色               'invert': 反白显示       #    
#            'purple':  紫红色        'purple':  紫红色            'hide': 不可见          #    
#            'cyan':  青蓝色          'cyan':  青蓝色                                     #
#            'white':  白色           'white':  白色                                     #
#########################################################################################
\"""
"""


def default_template(term, template):
    # default
    res = f'TEMPLATE = "{template}"\n'
    for t in term:
        res += '''%s_default = ""\n\n''' % t
    return res


def lib_template():
    return f"""from datetime import datetime
from colour_printing import Mode, Fore, Back\n
get_time = lambda: datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S.%f')[:-3]\n
"""


def tip(msg, colour='purple'):
    stream.write(cword(f"[*]Tip>> {msg}", fore=colour))
    stream.write('\n')


def term_template(term, kw: dict):
    style = kw.get(term, term_model(term))
    DEFAULT = style.get('DEFAULT')
    fore = style.get('fore')
    back = style.get('back')
    mode = style.get('mode')
    return f"""
        "{term}": {lk}
            "DEFAULT": {f'"{DEFAULT}"' if DEFAULT else f"{term}_default"},
            "fore": {f'"{fore}"' if fore else dy},
            "back": {f'"{back}"' if back else dy},
            "mode": {f'"{mode}"' if mode else dy}
        {rk},
"""


def term_model(term):
    return {term: {
        "DEFAULT": "",
        "fore": "",
        "back": "",
        "mode": ""
    }}


def level_template(level_list, term_list, config):
    res = ""
    for level in level_list:
        res += level + " = {"
        for term in term_list:
            res += term_template(term, config.get(level, term_model(term)))
        res += "}\n"
    return res


def new_pyfile_template(config):
    term = config['term']
    template = config['template']
    level_list = config['level_list']
    res = header
    # lib
    res += lib_template()
    res += default_template(term, template)
    # style
    res += level_template(level_list, term, config)
    return res


def create_py_file(file_path, level_list, term, template):
    config = dict(level_list=level_list, term=term, template=template)
    if os.path.exists(file_path):
        confirm = input(cword(f"[*]Tip>> 该配置文件已存在,确认要覆盖吗?\n"
                              f"(如果覆盖则会保留原有的且是新模板中所需的配置)[Y/n]:", fore='yellow'))
        if confirm != "Y":
            sys.exit(0)
        cfg = Config()
        cfg.from_pyfile(file_path)
        config.update(cfg)
    ns = new_pyfile_template(config)
    with open(file_path, 'w')as f:
        f.write(ns)
    tip(f'创建配置模板文件完成-->  {file_path}', colour='green')
    return 0


def execute():
    args = parser.parse_args()
    template = args.template
    name = args.filename
    new_level = args.newLevel
    # template
    if template:
        term = re.findall(r'(?<=\{)[^}]*(?=\})+', template)
        if "message" not in term:
            tip('template muse have {message} ! ', colour='red')
            sys.exit(2)
        for t in term:
            if t.strip() == '':
                tip(f'Unknown {{}} in " {template} " ', colour='red')
                sys.exit(2)
    # filepath
    name = name if name.endswith('.py') else name + '.py'
    file_path = f'{os.getcwd()}/{name}'
    # level
    level_list = ['INFO', 'SUCCESS', 'WARNING', 'ERROR', 'DEBUG']
    if new_level:
        level_list += new_level.split(" ")
    tip(u'创建配置模板文件中....')
    code = create_py_file(file_path, level_list, term, template)
    sys.exit(code)



STYLE = {
    'fore':
        {  # 前景色
            'black': 30,  # 黑色
            'red': 31,  # 红色
            'green': 32,  # 绿色
            'yellow': 33,  # 黄色
            'blue': 34,  # 蓝色
            'purple': 35,  # 紫红色
            'cyan': 36,  # 青蓝色
            'white': 37,  # 白色
        },

    'back':
        {  # 背景
            'black': 40,  # 黑色
            'red': 41,  # 红色
            'green': 42,  # 绿色
            'yellow': 43,  # 黄色
            'blue': 44,  # 蓝色
            'purple': 45,  # 紫红色
            'cyan': 46,  # 青蓝色
            'white': 47,  # 白色
        },

    'mode':
        {  # 显示模式
            'normal': 0,  # 终端默认设置
            'bold': 1,  # 高亮显示
            'underline': 4,  # 使用下划线
            'blink': 5,  # 闪烁
            'invert': 7,  # 反白显示
            'hide': 8,  # 不可见
        },

    'default':
        {
            'end': 0,
        },
}


def setting(mode='', fore='', back=''):
    """
    linxu:转义序列以ESC开头，即ASCII码下的\033
               格式: \033[显示方式;前景色;背景色m
    """
    mode = '%s' % STYLE['mode'][mode] if STYLE['mode'].get(mode) else ''

    fore = '%s' % STYLE['fore'][fore] if STYLE['fore'].get(fore) else ''

    back = '%s' % STYLE['back'][back] if STYLE['back'].get(back) else ''

    style = ';'.join([s for s in [mode, fore, back] if s])

    style = '\033[%sm' % style if style else ''

    end = '\033[%sm' % STYLE['default']['end'] if style else ''

    return style, end


class Fore(dict):
    BLACK = 'black'
    RED = 'red'
    GREEN = 'green'
    YELLOW = 'yellow'
    BLUE = 'blue'
    PURPLE = 'purple'
    CYAN = 'cyan'
    WHITE = 'white'


class Back(dict):
    BLACK = 'black'
    RED = 'red'
    GREEN = 'green'
    YELLOW = 'yellow'
    BLUE = 'blue'
    PURPLE = 'purple'
    CYAN = 'cyan'
    WHITE = 'white'


class Mode(dict):
    NORMAL = 'normal'
    BOLD = 'bold'
    UNDERLINE = 'underline'
    BLINK = 'blink'
    INVERT = 'invert'
    HIDE = 'hide'



from colour_printing.style import setting, Fore, Mode, Back


def cprint(*args, **kwargs):
    """
    :param args: 字符
    :param kwargs: 颜色参数
    :return: list 彩色字符
    sep=' ', end='\n', file=None
    """
    sep = kwargs.pop('sep', " ")
    end = kwargs.pop('end', '\n')
    file = kwargs.pop('file', None)
    str_temp = cword(*args, **kwargs)
    if not isinstance(str_temp, list):
        str_temp = [str_temp]
    if file:
        print(*args, sep=sep, end=end, file=file)
    else:
        print(*str_temp, sep=sep, end=end, file=file)


def cword(*args, **kwargs):
    """
    :param args: 字符
    :param kwargs: 颜色参数
    :return: list 彩色字符
    """
    mode = kwargs.get('mode')
    fore = kwargs.get('fore', Fore.CYAN)
    back = kwargs.get('back')
    style = setting(mode=mode, fore=fore, back=back)
    str_temp = []
    for s in args:
        str_temp.append(f'{style[0]}{s}{style[1]}')
    return str_temp[0] if len(str_temp) == 1 else str_temp
